      subroutine make_rnums()
      use math_blocks
      use rte_blocks
      use header_types
      implicit none

      real*4, dimension(1:nr) :: arr, phi, uni
c
c-------- RNs for all phase functions 
c
c-------- (STR1) Step length DEL: Exponential in [0,Inf], P[x]=Exp[-x]
      vsl_errcode = vsrngexponential(meth1,str1,nr,del, rzero, ru )
      call check_vsl_error(vsl_errcode)
c-------- (STR2) Cosine of polar angle CTH
c     Specific to each phase function
c-------- (STR3) Azimuthal angle PHI: Uniform in [0,2*PI]      
      vsl_errcode =     vsrnguniform(meth3,str3,nr,phi, rzero, pi2)
      call check_vsl_error(vsl_errcode)
      call vssin(nr,phi,sph)
      call vscos(nr,phi,cph)
c-------- (STR4) Absorption probability: Binary
      vsl_errcode =   virngbernoulli(meth4,str4,nr,bin,albedo)
      call check_vsl_error(vsl_errcode)

c-------- a) PP=HG 

      IF(PhaseFunction .eq. 'HG') THEN

      vsl_errcode = vsrnguniform(meth2,str2,nr,uni,rzero,ru)
      call check_vsl_error(vsl_errcode)

      cth = ru - g + twog*uni
      call vsinv(nr,cth,arr)
      arr = arr*omgs
      call vssqr(nr,arr,cth)
      cth = opgs - cth
      cth = cth / twog
      call check_cosines(nr,cth)
      call vssqr (nr,cth,arr)
      arr = ru - arr
      call vssqrt(nr,arr,sth)

c-------- b) PP=HG0 (0<|g|<0.01)

      ELSE IF(PhaseFunction .eq. 'HG0') THEN

      vsl_errcode = vsrnguniform(meth2,str2,nr,uni,rzero,ru)
      call check_vsl_error(vsl_errcode)

      cth = ru - uni
      call vsmul(nr,uni,cth,arr)
      arr = 6.0*g*arr
      cth = two*uni - ru + arr
      call check_cosines(nr,cth)
      call vssqr (nr,cth,arr)
      arr = ru - arr
      call vssqrt(nr,arr,sth)

c-------- c) PP=ISOTROPIC

      ELSE IF(PhaseFunction .eq. 'ISO') THEN

      vsl_errcode = vsrnguniform(meth2,str2,nr,cth,-ru,ru)
      call check_vsl_error(vsl_errcode)

      call vssqr (nr,cth,arr)
      arr = ru - arr
      call vssqrt(nr,arr,sth)

      ELSE IF(PhaseFunction .eq. 'EXP') THEN

      vsl_errcode = vsrnguniform(meth2,str2,nr,uni,rzero,ru)
      call check_vsl_error(vsl_errcode)
      cth = tshb*uni + ebm
      call vsln(nr,cth,arr)
      cth = arr / beta
      call check_cosines(nr,cth)
      call vssqr (nr,cth,arr)
      arr = ru - arr
      call vssqrt(nr,arr,sth)

      ELSE IF(PhaseFunction .eq. 'RAY') THEN

         write(*,*) 'MAKE_RNUMS: PhaseFunction DEL not yet implemented'
         pause

      ELSE IF(PhaseFunction .eq. 'DEL') THEN

         write(*,*) 'MAKE_RNUMS: PhaseFunction DEL not yet implemented'
         pause

      ELSE

         write(*,*) 'MAKE_RNUMS: Unrecognized PhaseFunction'
         pause

      END IF


      return
      end
